;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x2464ecb55f9563dc0b2a49019a18bab1 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xa53340fdbca2523b3b8b729660474a83 . "./code.cg2js/cg-printing.fasl")
(#x1a08b0b40123470b22fb3f63e8c444f0 . "./code.cg2js/cg-caret.fasl")
(#xe3f23d66c3c173483a6111b8ba668b60 . "./code.cg2js/cg-item-list.fasl")
(#xc9eba8861495b557e1f5080cb84ec6f4 . "./code.cg2js/cg-progress-indicator.fasl")
(#x1caabb377a5180751c85fc12ce1f740c . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xc9c355d8e28521dccb4eb6231027cd67 . "./code.cg2js/cg-outline.fasl")
(#xf8c438573972f32f39f2ff34f00dc9d8 . "./code.cg2js/cg-palette.fasl")
(#x310b8fb5219269695393304b5b022484 . "./code.cg2js/cg-selected-object.fasl")
(#xb4d7fb8690103721ee3d01b80014c35b . "./code.cg2js/cg-status-bar.fasl")
(#xdfa5a2a7958ec1ff750187cbeac0a9b7 . "./code.cg2js/cg-wrap-string.fasl")
(#xd9b37fbe92754f0b72e00be6457ef559 . "./code.cg2js/cg-fill-texture.fasl")
(#xa46728bdaada9584763e1fff279fd6b8 . "./code.cg2js/cg-layout.fasl")
(#x341547c767ea55ebec55e14c151ede12 . "./code.cg2js/cg-timer.fasl")
(#xe16483284fda01717d2770b1032a0d29 . "./code.cg2js/cg-dialog-item.fasl")
(#x0603fdd71fef4e3831ee3fcb0135b400 . "./code.cg2js/cg-button.fasl")
(#xaafb4ed8888f179db5ec0f552172ac35 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x069492cda02ddf4f17c32ae1546ce2ad . "./code.cg2js/cg-tray.fasl")
(#xc176e674ce04bca3f405a4d0a9ad2a3c . "./code.cg2js/cg-plot-widget.fasl")
(#xfa606a2a1698dc9671168b504df7e375 . "./code.cg2js/cg-pixmap.fasl")
(#x7751b02140d11921e3826ae948271c54 . "./code.cg2js/cg-font-dialog.fasl")
(#x276c08bf32377f39f25887530469a5e0 . "./code.cg2js/cg-os-widget.fasl")
(#x1d50bf98a2ee646ca9b274ef7f381bfb . "./code.cg2js/cg-text-or-combo.fasl")
(#x6e18a5c2d10c440afe5092753d0ee367 . "./code.cg2js/cg-trackbar.fasl")
(#x3190f86af58deafa21496e859d706730 . "./code.cg2js/cg-up-down-control.fasl")
(#x6d7598f6794b787fb18a2e1b2e694deb . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xbc72324692cc31d887f37b826aafc533 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x39984ca5a5bf9832053ae0f801a3896f . "./code.cg2js/cg-intersect-posbox.fasl")
(#x0aa594dfa891e0b214257570efda914f . "./code.cg2js/cg-bitmap-stream.fasl")
(#xc30cb82c3cd1a82e1cd3f6073c55e0f0 . "./code.cg2js/cg-chart-widget.fasl")
(#x0a05e1385a9357514d79272c7ae6f823 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x5794dec2a7d3a17f926242174d63054e . "./code.cg2js/cg-group-box.fasl")
(#x2f6a4792c3077f4faea05cdd41fff4a7 . "./code.cg2js/cg-static-text.fasl")
(#xf01a1ab4b3ed17f59ee37daebca80424 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x4ba2a3b0ac1bbb89db01aca3dbf4ff3c . "./code.cg2js/cg-color-dialog.fasl")
(#x563e4dea5d7e8762da6a21626e8d25c2 . "./code.cg2js/cg-intersect.fasl")
(#x0ecd5234496df0219ddb800a1d412fc0 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x1cdaecb04507d3622f4ecea0a9c21d5b . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x070d36141ceb35b5e08c3bafab4acb64 . "./code.cg2js/cg-html-dialog.fasl")
(#x1d1f8c35ada4d3d80535492a54178f6f . "./code.cg2js/cg-lettered-menu.fasl")
(#xa72fc29920e14d1b868adf8c1e649486 . "./code.cg2js/cg-class-grid.fasl")
(#xd594715a1fc4ed52c4119862933a8874 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x0d3e062d7bb0bb53c404f851e9aa0901 . "./code.cg2js/cg-calendar.fasl")
(#xc5046756bf3e78f9bedec463df477de8 . "./code.cg2js/cg-editable-text.fasl")
(#x00f36c44ff7c01a61125d7c35985c3ba . "./code.cg2js/cg-property.fasl")
(#x2bf0834c120d18a8addddeb172a54260 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xd6266226a0776ba464642ff116a4e792 . "./code.cg2js/cg-grid-widget.fasl")
(#xca5e48ca052688df4309d8ba81a2f2e0 . "./code.cg2js/cg-paren-matching.fasl")
(#x6de363d068a20e78976dc270e3ea98ba . "./code.cg2js/cg-dev.fasl")
(#x679aca417472773cba5ed118cd809f2b . "./code.cg2js/cg-drawable.fasl")
(#x7a171b0abe0591261d0ded27cc0e52bd . "./code.cg2js/cg-curve.fasl")
(#x9d81abaef90458d8bae03c3d7b7f6a57 . "./code.cg2js/cg-lisp-widget.fasl")
(#x3c81da997c4fd5032f50820b745b894b . "./code.cg2js/cg-dde-utils.fasl")
(#x5c2b92d0748e8b4bde9342ef5eb8f6c9 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x28771dabff444666e42c7a31887c35c3 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xe94a1694719e6cb0c890ed6174a61a72 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x91cc3860572bb468e2c6b172addd88d1 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x105a3da9edbf5a48b5ef9be371976239 . "./code.cg2js/cg-lamp.fasl")
(#xf3cc9ec7e9a1e5a5425cf5711c36421b . "./code.cg2js/cg-common-control.fasl")
(#x40e9b0c82b65d23185a81adc03af0658 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x9d404f98b1a31480cdb8c6b3200be58a . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xd830444506b13e7c6db65aba6b00a844 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x62ae2981a081ded388cddc4708af05a4 . "./code.cg2js/cg-project-window.fasl")
(#xf1e73eb0c51d9fafc0ebc9292b278594 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xb94a30f37a7e7c1a8fda5fc1060bd481 . "./code.cg2js/cg-class-support.fasl")
(#x994d2daa8d6fb88b77a9c809b93623dd . "./code.cg2js/cg-multi-picture-button.fasl")
(#xa4946e89cb71b83b71230646f28a25aa . "./code.cg2js/cg-yes-no-list.fasl")
(#x592d57d9cfb6352cbe2c9e758e7643c1 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x578a775725eba873101c4d672afe594e . "./code.cg2js/cg-string-dialog.fasl")
(#xb7cc3d1ad1e8fda8f5b160084f09c62c . "./code.cg2js/cg-menu-tooltip.fasl")
(#x76b0b375562d8eac81573eb28120ecd7 . "./code.cg2js/cg-dropping-outline.fasl")
(#xeab02fe273fc2d16ea16a6fa0cc11278 . "./code.cg2js/cg.fasl")
(#x79fb2a311ce2ed77c338041f238df2de . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xa57953d529bd71413f2326d897161831 . "./code.cg2js/cg-scaling-stream.fasl")
(#x30327db72c14f8edcbc53b9c6d7c345a . "./code.cg2js/cg-yes-no-string.fasl")
(#x98972ddf624a78544bcccc2739e09e58 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xb106208cb02a9f6be491d4bc13922882 . "./code.cg2js/cg-web-browser.fasl")
(#x45ecb9dcbcdb114efd959eca4ef33c00 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x60d47bab36ac117a2b909fd7dd2d60ca . "./code.cg2js/cg-drag-and-drop.fasl")
(#xa8786d54c1f75fe2bae9b90e424a52c6 . "./code.cg2js/cg-message-dialog.fasl")
(#x2472f16ea7b9101ac72718e0be491eee . "./code.cg2js/cg-comtab.fasl")
(#x31c5ff8720874c14cb8a9c941b27610c . "./code.cg2js/cg-base.fasl")
(#x9e84f989938ea2fca9f3fd4c19c31a47 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x0258e1518a989feb8b273499aa2f5972 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x20d6d54fa8953eb4c2ce73a5a1ba1dc3 . "./code.cg2js/cg-utility-dialog.fasl")
(#x16bb79f94169562e09441dc292bfbb2f . "./code.cg2js/cg-object-editor.fasl")
(#xadb6de3f569260cdac7d852e1467c122 . "./code.cg2js/cg-edit-in-place.fasl")
(#xd3b936d793b4bb21f5b5e4e67a7fbc99 . "./code.cg2js/cg-radio-button.fasl")
(#x1bad6faac15a0f22084d7d223a262e1b . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xe8064987f6f678b78bdbb9189430ef72 . "./code.cg2js/cg-nodes.fasl")
(#x0fbc82737e3c7b99d073130adb07f6ff . "./code.cg2js/cg-get-position.fasl")
(#xc668f64f8703ef737b6658ba88fba4ac . "./code.cg2js/cg-directory-dialog.fasl")
(#xac7b4a0407c535abe8abc1d56a9bbccc . "./code.cg2js/cg-lisp-text.fasl")
(#x6ad552253022d1178dfaee2c97c252c5 . "./code.cg2js/cg-combo-box.fasl")
(#xd63c034b83f5cf7ccd8ea512b44281e5 . "./code.cg2js/cg-text-widget.fasl")
(#xb47c270cbe93d324b9b30bf90b44f94f . "./code.cg2js/cg-list-view.fasl")
(#xbbbe3b1bbcdd7a83028b2bfb31fbc69f . "./code.cg2js/cg-graphics-context.fasl")
(#x34f476ad01e8d7d888f3c226b4421cbe . "./code.cg2js/cg-get-pixmap.fasl")
(#xf9d9169367bba981325dfec41042867b . "./code.cg2js/cg-menu.fasl")
(#xfe4a63f4e7ed3bd36e8e9aed43ec3304 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x1c04435254c7b1dafa78e243d677ec68 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x3409d732f437219f81130075351bd231 . "./code.cg2js/cg-tooltip.fasl")
(#xe55b5f1ea4bd07418c54cc88ace81638 . "./code.cg2js/cg-html-widget.fasl")
(#x0d855c1b99008f068c5ed1817f6c6bb7 . "./code.cg2js/cg-choice-list.fasl")
(#x664f1363d96b1027100d7a1d4e528424 . "./code.cg2js/cg-icon.fasl")
(#x7accf967576da1f0b9d77f682a738c7d . "./code.cg2js/cg-acache.fasl")
(#x6d307e6425fe29a688cfd623b9da43b1 . "./code.cg2js/cg-template-string.fasl")
(#x9a910991509ffde04e111376cf73220b . "./code.cg2js/cg-bitmap-pane.fasl")
(#x0ff5261f2877df61e9d1e21c39b9d61e . "./code.cg2js/cg-toggling-widget.fasl")
(#xd83f4b533e2e36c167ada1125539fa2b . "./code.cg2js/cg-tab-control.fasl")
(#x0cf589947eeb37519852bedadb39a1f8 . "./code.cg2js/cg-split-bar.fasl")
(#x005066adef624c96de9467cfe6d7035a . "./code.cg2js/cg-picture-widget.fasl")
(#x080ceb7a44c65928cc492bd1ffa6c1cf . "./code.cg2js/cg-player.fasl")
(#x34a1e6a04e143dd10a0104e11e7ab0fa . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xe14d04fa12b5648282e45477f9eafb2a . "./code.cg2js/cg-choose-printer.fasl")
(#x5c48dc98aba9f10834915e58a37c97d5 . "./code.cg2js/cg-os-window.fasl")
(#x39d0c19cc7d65136cab2e749952c3571 . "./code.cg2js/cg-scroll-bar.fasl")
(#xec4a94099f114ea7f53f26f156609b8c . "./code.cg2js/cg-clipboard.fasl")
(#xcb58a7950a119a956a02f36db9060145 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x43eb78ab54d5a24dc61f5a8636aebeed . "./code.cg2js/cg-hotspot.fasl")
(#xac230c76a52203699f4df30abc0368a7 . "./code.cg2js/cg-check-box.fasl")
(#xe907e26b9dee5be68576dd48d2d915cf . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xf6e864575306cd79e9a20c9bbb6b8e0f . "./code.cg2js/cg-file-dialog.fasl")
(#xb8b16192a73cb77f0f74a6abaa023e65 . "./code.cg2js/cg-toolbar.fasl")
(#xf25afa29951d26e5bf8f3415f095530a . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xaf95b9dcc40f2d92307eba4efea98eef . "./code.cg2js/cg-chart-or-plot.fasl")
(#x2e06d6f875ad07f1b0fc5afccd680261 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xbbee181e26afdc96b5bb6b6b59cd78dc . "./code.idejs/ide-project.fasl")
(#x5f949792c66b671d474d9885e019970b . "./code.idejs/ide-intro.fasl")
(#xe22bceafa349cf3787a0b8beb5bdb40f . "./code.idejs/ide-find-in-files.fasl")
(#xff1fe7ccb572d2ef80c78ae616e4e93b . "./code.idejs/ide-grapher.fasl")
(#x3d10decc83b0c9b5c728b71b47a716fb . "./code.idejs/ide-inspector.fasl")
(#x21d5fa31a6d8006164f699db2b372e60 . "./code.idejs/ide-editor.fasl")
(#x023cc2cd21a519fdfd727d50edd5ef41 . "./code.idejs/ide-window-list.fasl")
(#xb4ac8769e94960f4a4a71ce6cf3eb49f . "./code.idejs/ide-process-browser.fasl")
(#x8fb8b660ea2c8f45d873d1ac75159df5 . "./code.idejs/ide-patch-dialog.fasl")
(#x9a53f698b0768aa4f80510f633457bcf . "./code.idejs/ide-class-grapher.fasl")
(#x0b27b73c848c4bfd7420c9a0cb5491c7 . "./code.idejs/ide-base.fasl")
(#xded7f9ff8f8da0de8ce368374a606f6a . "./code.idejs/ide.fasl")
(#x10aaf8e5878083bdee99c59a97b741b8 . "./code.idejs/ide-trace.fasl")
(#xa5cbed77eee771edcb49f24aef992ac9 . "./code.idejs/ide-form.fasl")
(#x8fc45a2192e318f9097bae2852ad0092 . "./code.idejs/ide-find-definitions.fasl")
(#x90f827676948c6b3e5cc649025f2b3b6 . "./code.idejs/ide-apropos.fasl")
(#xb3f0714ca995e0acb127c12ab84755ed . "./code.idejs/ide-class-browser.fasl")
(#xaf7ba1a1c24f187fccaabf1ba2165319 . "./code.idejs/ide-navigator.fasl")
(#x841eb12c424572773be136ad7e429629 . "./code.idejs/ide-clipboard.fasl")
(#x102ada4fd759f989a9733f91320e2ae4 . "./code.idejs/ide-acache.fasl")
(#x766918f0dfbb7b641482059a808d6c12 . "./code.idejs/ide-keyshort.fasl")
(#x14d1f492c6bc663c16dd686ef91db155 . "./code.idejs/ide-package-list.fasl")
(#x92e980b90784e1bc39dca2477967a124 . "./code.idejs/ide-dev.fasl")
(#xf3f59eb0bf604c0b34476c9897472f9c . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
